#include "array.h"

#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

void swap(int32_t *a, int32_t *b) {
    int32_t tmp = *a;
    *a = *b;
    *b = tmp;
}

int32_t *create_array(uint32_t array_length) {
    int32_t *array = malloc(array_length * sizeof(int32_t));
    return array;
}

void init_array(int32_t *array, uint32_t array_length) {
    for (uint32_t i = 0; i < array_length; i += 1) {
        array[i] = i;
    }

    permute_array_randomly(array, array_length);
}

void display_array(int32_t *array, uint32_t array_length) {
    printf("{");

    for (uint32_t i = 0; i < array_length; i += 1) {
        printf("%d", array[i]);

        if (i != array_length - 1)
            printf(",");
    }

    printf("}\n");
}

void permute_array_randomly(int32_t *array, uint32_t array_length) {
    for (uint32_t i = 0; i < array_length; i += 1) {
        uint32_t index_1 = rand() % (int32_t)array_length;
        uint32_t index_2 = rand() % (int32_t)array_length;
        swap(&(array[index_1]), &(array[index_2]));
    }
}

void permute_array_cyclic(int32_t *array, uint32_t array_length, uint32_t n) {
    int32_t *tmp = (int32_t *)malloc(sizeof(array[0]) * array_length);

    for (uint32_t i = 0; i < array_length; i += 1) {
        tmp[(i + n) % array_length] = array[i];
    }

    for (uint32_t i = 0; i < array_length; i += 1) {
        array[i] = tmp[i];
    }

    free(tmp);
}

uint32_t find_lowest_value_index(int32_t *array, uint32_t array_length) {
    uint32_t lowest_index = 0;

    for (uint32_t i = 1; i < array_length; i += 1) {
        if (array[i] < array[lowest_index]) {
            lowest_index = i;
        }
    }

    return lowest_index;
}

void place_lowest_value_last(int32_t *array, uint32_t array_length) {
    uint32_t lowest_index = find_lowest_value_index(array, array_length);
    int32_t tmp = array[array_length - 1];
    array[array_length - 1] = array[lowest_index];
    array[lowest_index] = tmp;
}

void insertion_sort_desc(int32_t *array, uint32_t array_length) {
    uint32_t i = 1;

    while (i < array_length) {
        uint32_t j = i;

        while (j > 0 && array[j - 1] < array[j]) {
            int32_t tmp = array[j];
            array[j] = array[j - 1];
            array[j - 1] = tmp;
            j -= 1;
        }

        i += 1;
    }
}

uint32_t count_elements_lower_than(int32_t *array, uint32_t array_length, uint32_t n) {
    uint32_t count = 0;

    for (uint32_t i = 0; i < array_length; i += 1) {
        if (array[i] < n) {
            count += 1;
        }
    }

    return count;
}

int32_t *sum(int32_t *operand_1, int32_t *operand_2, uint32_t array_length) {
    int32_t *result = create_array(array_length);

    for (uint32_t i = 0; i < array_length; i += 1) {
        result[i] = operand_1[i] + operand_2[i];
    }

    return result;
}

void multiply_by_n(int32_t *array, uint32_t array_length, uint32_t n) {
    for (uint32_t i = 0; i < array_length; i += 1) {
        array[i] *= n;
    }
}

double *convert_int32_t_array_to_double(int32_t *array, uint32_t array_length) {
    double *result = malloc(array_length * sizeof(double));

    for (uint32_t i = 0; i < array_length; i += 1) {
        result[i] = (double)array[i];
    }

    return result;
}
