/**
 * @file main.c
 * @author Florian Burgener (florian.burgener@etu.hesge.ch)
 * @brief Travail pratique sur les allocations dynamiques.
 * @version 1.0
 * @date 2021-11-09
 *
 * @copyright Copyright (c) 2021
 *
 */
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "array.h"

const uint32_t INTERACTIVE_BEHAVIOR = 0;
const uint32_t AUTOMATIC_BEHAVIOR = 1;

void execute_tasks(uint32_t behavior) {
    // 1.
    uint32_t array_length;

    if (behavior == 0) {
        printf("Array Size: ");
        scanf("%d", &array_length);
    } else {
        array_length = rand() % 10 + 11;
    }

    int32_t *tab = create_array(array_length);

    // 2.
    init_array(tab, array_length);

    // 3.
    printf("\n3.\n");
    display_array(tab, array_length);

    // 4.
    printf("\n4.\n");
    permute_array_randomly(tab, array_length);
    display_array(tab, array_length);

    // 5.
    printf("\n5.\n");
    permute_array_cyclic(tab, array_length, 3);
    display_array(tab, array_length);

    // 6.
    printf("\n6.\n");
    place_lowest_value_last(tab, array_length);
    display_array(tab, array_length);

    // 7.
    printf("\n7.\n");
    insertion_sort_desc(tab, array_length);
    display_array(tab, array_length);

    // 8.
    printf("\n8.\n");
    uint32_t low_value;

    if (behavior == 0) {
        printf("Enter a number (counts all numbers below your number): ");
        scanf("%d", &low_value);
    } else {
        low_value = rand() % array_length;
    }

    uint32_t count = count_elements_lower_than(tab, array_length, low_value);
    printf("%d numbers are lower than %d\n", count, low_value);

    // 9.
    printf("\n9.\n");
    int32_t *tab_two = create_array(array_length);
    init_array(tab_two, array_length);

    printf("  ");
    display_array(tab, array_length);
    printf("+ ");
    display_array(tab_two, array_length);

    int32_t *tab_three = sum(tab, tab_two, array_length);
    printf("= ");
    display_array(tab_three, array_length);

    // 10.
    printf("\n10.\n");
    int32_t *tab_four = create_array(array_length);
    init_array(tab_four, array_length);
    display_array(tab_four, array_length);

    uint32_t multiplicator;
    if (behavior == 0) {
        printf("Enter a multiplicator (which will be applied to each number): ");
        scanf("%d", &multiplicator);
    } else {
        multiplicator = rand() % 10 + 1;
    }

    multiply_by_n(tab_four, array_length, multiplicator);
    display_array(tab_four, array_length);

    // 11.
    printf("\n11.\n");
    double *tab_five = convert_int32_t_array_to_double(tab, array_length);

    free(tab);
    free(tab_two);
    free(tab_three);
    free(tab_four);
    free(tab_five);
}

int main() {
    srand(time(NULL));
    execute_tasks(INTERACTIVE_BEHAVIOR);

    printf("Do you want to perform task 11? [Y/n] ");
    char choice;
    while ((choice = getchar()) != '\n' && choice != EOF) {
    }
    scanf("%c", &choice);

    if (choice == 'n') {
        printf("Exiting...\n");
        return EXIT_SUCCESS;
    }

    for (uint32_t i = 0; i < 100; i += 1) {
        execute_tasks(AUTOMATIC_BEHAVIOR);
    }

    return EXIT_SUCCESS;
}
